﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.DragDrop;
using Telerik.Windows.Controls;
using System.Windows.Media.Imaging;
using Telerik.Windows.Controls.DragDrop;

namespace TileViewWithNewDragAndDrop
{
	public partial class MainPage : UserControl
	{
		private RadTileView source;
		private RadTileView destination;

		public MainPage()
		{
			InitializeComponent();

			DragDropManager.AddDragInitializeHandler(this.firstTileView, OnDragInitialized);
			DragDropManager.AddDropHandler(this.firstTileView, OnDrop);
			DragDropManager.AddDragInitializeHandler(this.secondTileView, OnDragInitialized);
			DragDropManager.AddDropHandler(this.secondTileView, OnDrop);
			DragDropManager.AddDragDropCompletedHandler(this.firstTileView, OnDragCompleted);
			DragDropManager.AddDragDropCompletedHandler(this.secondTileView, OnDragCompleted);

			for (int i = 0; i < 5; i++)
			{
				this.firstTileView.Items.Add(i);
			}

			for (int i = 5; i < 10; i++)
			{
				this.secondTileView.Items.Add(i);
			}

			firstTileView.ItemContainerGenerator.StatusChanged += new EventHandler(ItemContainerGenerator_StatusChanged);
		}

		void ItemContainerGenerator_StatusChanged(object sender, EventArgs e)
		{
			if (this.firstTileView.ItemContainerGenerator.Status == Telerik.Windows.Controls.Primitives.GeneratorStatus.ContainersGenerated)
			{
				this.Dispatcher.BeginInvoke(new Action(() =>
				{
					for (int i = 0; i < this.firstTileView.Items.Count; i++)
					{
						var container = this.firstTileView.Items[i] as RadTileViewItem;
						if (container != null)
						{
							RadDragAndDropManager.SetAllowDrag(container, true);
							DragDropManager.SetAllowDrag(container, true);
						}
					}
				}));
			}
		}

		private void OnDragCompleted(object sender, DragDropCompletedEventArgs args)
		{
			var tileView = sender as RadTileView;
			var item = args.Data;
			if (item != null && tileView != null)
			{
				if (this.destination != tileView)
				{
					tileView.Items.Remove(item);
				}
				else
				{
					var container = tileView.ItemContainerGenerator.ContainerFromItem(item) as RadTileViewItem;
					if (container != null)
					{
						container.Opacity = 1;
					}
				}
			}
		}

		private void OnDragInitialized(object sender, DragInitializeEventArgs args)
		{
			var tileView = sender as RadTileView;
			var tileViewItem = args.OriginalSource as RadTileViewItem;
			if (tileViewItem != null && tileView != null)
			{
				this.source = tileView;
				args.Data = tileView.ItemContainerGenerator.ItemFromContainer(tileViewItem);
				WriteableBitmap writeableBitmap = new WriteableBitmap(tileViewItem, null);
				Image draggingImage = new Image() { Source = writeableBitmap };
				args.DragVisual = draggingImage;
				tileViewItem.Opacity = 0;

			}
			args.AllowedEffects = DragDropEffects.Copy;
			args.Handled = true;
		}

		private void OnDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs args)
		{
			var tileView = sender as RadTileView;
			var item = args.Data;
			if (item != null && tileView != null && this.source != tileView)
			{
				tileView.Items.Add(item);
			}
			this.destination = tileView;
		}
	}
}
